@@ -1,6 +1,27 @@
-#include <string.h>
+/* -*- mode: C; indent-tabs-mode: t; tab-width: 8; c-basic-offset: 2; -*- */
+
+/*
+ * This file is part of Seed, the GObject Introspection<->Javascript bindings.
+ *
+ * Seed is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation, either version 3 of
+ * the License, or (at your option) any later version.
+ * Seed is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Lesser General Public License for more details.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with Seed.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Copyright (C) Robert Carr 2009 <carrr@rpi.edu>
+ */
+
+// TODO: Don't use GNU libc extension canonicalize_file_name, don't use realpath?
+#define _GNU_SOURCE
 
 #include <gio/gio.h>
+#include <string.h>
 
 #include "seed-private.h"
 
@@ -58,13 +79,13 @@ GHashTable *file_imports;
 */
 
 static gboolean
-seed_gi_importer_is_init (GIFunctionInfo *info)
+seed_gi_importer_is_init (GIFunctionInfo * info)
 {
-  if (strcmp (g_base_info_get_name ((GIBaseInfo *) info), "init"))
+  if (g_strcmp0 (g_base_info_get_name ((GIBaseInfo *) info), "init"))
     {
       return FALSE;
     }
-  if (g_callable_info_get_n_args ((GICallableInfo *)info) != 2)
+  if (g_callable_info_get_n_args ((GICallableInfo *) info) != 2)
     return FALSE;
 
   return TRUE;
@@ -73,8 +94,8 @@ seed_gi_importer_is_init (GIFunctionInfo *info)
 static void
 seed_gi_importer_handle_function (JSContextRef ctx,
 				  JSObjectRef namespace_ref,
-				  GIFunctionInfo *info,
-				  JSValueRef *exception)
+				  GIFunctionInfo * info,
+				  JSValueRef * exception)
 {
   if (!seed_gi_importer_is_init (info))
     seed_gobject_define_property_from_function_info (ctx,
@@ -84,8 +105,8 @@ seed_gi_importer_handle_function (JSContextRef ctx,
     {
       JSObjectRef init_method;
 
-      init_method = JSObjectMake(ctx, gobject_init_method_class,
-				 g_base_info_ref ((GIBaseInfo *)info));
+      init_method = JSObjectMake (ctx, gobject_init_method_class,
+				  g_base_info_ref ((GIBaseInfo *) info));
       seed_object_set_property (ctx, namespace_ref, "init", init_method);
     }
 }
@@ -99,27 +120,27 @@ seed_gi_importer_handle_function (JSContextRef ctx,
 static void
 seed_gi_importer_handle_enum (JSContextRef ctx,
 			      JSObjectRef namespace_ref,
-			      GIEnumInfo *info,
-			      JSValueRef *exception)
+			      GIEnumInfo * info, JSValueRef * exception)
 {
   JSObjectRef enum_class;
-  gint num_vals, j;
+  guint num_vals, i, j;
+  gsize name_len;
+  glong value;
+  gchar *name;
+  GIValueInfo *val;
 
   enum_class = JSObjectMake (ctx, 0, 0);
   num_vals = g_enum_info_get_n_values (info);
   seed_object_set_property (ctx, namespace_ref,
-			    g_base_info_get_name ((GIBaseInfo *)info),
+			    g_base_info_get_name ((GIBaseInfo *) info),
 			    enum_class);
 
-  for (j = 0; j < num_vals; j++)
+  for (i = 0; i < num_vals; i++)
     {
-      GIValueInfo *val =
-	g_enum_info_get_value ((GIEnumInfo *) info, j);
-      gint value = g_value_info_get_value (val);
-      gchar *name =
-	g_strdup (g_base_info_get_name ((GIBaseInfo *) val));
-      gint name_len = strlen (name);
-      gint j;
+      val = g_enum_info_get_value ((GIEnumInfo *) info, i);
+      value = g_value_info_get_value (val);
+      name = g_strdup (g_base_info_get_name ((GIBaseInfo *) val));
+      name_len = strlen (name);
       JSValueRef value_ref;
 
       value_ref = JSValueMakeNumber (ctx, value);
@@ -148,14 +169,12 @@ seed_gi_importer_handle_enum (JSContextRef ctx,
 static void
 seed_gi_importer_handle_object (JSContextRef ctx,
 				JSObjectRef namespace_ref,
-				GIObjectInfo *info,
-				JSValueRef *exception)
+				GIObjectInfo * info, JSValueRef * exception)
 {
   GType type;
   JSClassRef class_ref;
 
-  type =
-    g_registered_type_info_get_g_type ((GIRegisteredTypeInfo *) info);
+  type = g_registered_type_info_get_g_type ((GIRegisteredTypeInfo *) info);
 
   if (type != 0)
     {
@@ -167,9 +186,7 @@ seed_gi_importer_handle_object (JSContextRef ctx,
       class_ref = seed_gobject_get_class_for_gtype (ctx, type);
 
       constructor_ref =
-	JSObjectMake (ctx,
-		      gobject_constructor_class,
-		      (gpointer) type);
+	JSObjectMake (ctx, gobject_constructor_class, (gpointer) type);
 
       seed_object_set_property (ctx, constructor_ref,
 				"type",
@@ -186,14 +203,13 @@ seed_gi_importer_handle_object (JSContextRef ctx,
 						      finfo);
 	      const gchar *fname =
 		g_base_info_get_name ((GIBaseInfo *) finfo);
-	      if (strstr (fname, "new_") == fname)
+	      if (g_strrstr (fname, "new_") == fname)
 		fname += 4;
-	      else if (!strcmp (fname, "new"))
+	      else if (!g_strcmp0 (fname, "new"))
 		fname = "c_new";
 
 	      seed_object_set_property (ctx,
-					constructor_ref,
-					fname, constructor);
+					constructor_ref, fname, constructor);
 	    }
 	  else if (!(flags & GI_FUNCTION_IS_METHOD))
 	    {
@@ -222,16 +238,14 @@ seed_gi_importer_handle_object (JSContextRef ctx,
 static void
 seed_gi_importer_handle_struct (JSContextRef ctx,
 				JSObjectRef namespace_ref,
-				GIStructInfo *info,
-				JSValueRef *exception)
+				GIStructInfo * info, JSValueRef * exception)
 {
   JSObjectRef struct_ref;
   JSObjectRef proto;
   gint i, n_methods;
   GIFunctionInfo *finfo;
 
-  struct_ref =
-    JSObjectMake (ctx, seed_struct_constructor_class, info);
+  struct_ref = JSObjectMake (ctx, seed_struct_constructor_class, info);
 
   n_methods = g_struct_info_get_n_methods (info);
 
@@ -247,11 +261,10 @@ seed_gi_importer_handle_struct (JSContextRef ctx,
 	  JSObjectRef constructor = JSObjectMake (ctx,
 						  gobject_named_constructor_class,
 						  finfo);
-	  const gchar *fname =
-	    g_base_info_get_name ((GIBaseInfo *) finfo);
+	  const gchar *fname = g_base_info_get_name ((GIBaseInfo *) finfo);
 	  if (g_str_has_prefix (fname, "new_"))
 	    fname += 4;
-	  else if (!strcmp (fname, "new"))
+	  else if (!g_strcmp0 (fname, "new"))
 	    fname = "c_new";
 
 	  seed_object_set_property (ctx, struct_ref, fname, constructor);
@@ -263,25 +276,25 @@ seed_gi_importer_handle_struct (JSContextRef ctx,
 	  (ctx, finfo, struct_ref, FALSE);
     }
 
-  proto = seed_struct_prototype (ctx, (GIBaseInfo *)info);
+  proto = seed_struct_prototype (ctx, (GIBaseInfo *) info);
   seed_object_set_property (ctx, struct_ref, "prototype", proto);
 
-  seed_object_set_property (ctx, namespace_ref, g_base_info_get_name ((GIBaseInfo *)info), struct_ref);
+  seed_object_set_property (ctx, namespace_ref,
+			    g_base_info_get_name ((GIBaseInfo *) info),
+			    struct_ref);
 }
 
 static void
 seed_gi_importer_handle_union (JSContextRef ctx,
 			       JSObjectRef namespace_ref,
-			       GIUnionInfo *info,
-			       JSValueRef *exception)
+			       GIUnionInfo * info, JSValueRef * exception)
 {
   JSObjectRef union_ref;
   JSObjectRef proto;
-  gint i, n_methods;
+  guint i, n_methods;
   GIFunctionInfo *finfo;
 
-  union_ref =
-    JSObjectMake (ctx, seed_struct_constructor_class, info);
+  union_ref = JSObjectMake (ctx, seed_struct_constructor_class, info);
 
   n_methods = g_union_info_get_n_methods (info);
 
@@ -299,17 +312,19 @@ seed_gi_importer_handle_union (JSContextRef ctx,
 	  (ctx, finfo, union_ref, FALSE);
     }
 
-  proto = seed_union_prototype (ctx, (GIBaseInfo *)info);
+  proto = seed_union_prototype (ctx, (GIBaseInfo *) info);
   seed_object_set_property (ctx, union_ref, "prototype", proto);
 
-  seed_object_set_property (ctx, namespace_ref, g_base_info_get_name ((GIBaseInfo *)info), union_ref);
+  seed_object_set_property (ctx, namespace_ref,
+			    g_base_info_get_name ((GIBaseInfo *) info),
+			    union_ref);
 }
 
 static void
 seed_gi_importer_handle_callback (JSContextRef ctx,
 				  JSObjectRef namespace_ref,
-				  GICallbackInfo *info,
-				  JSValueRef *exception)
+				  GICallbackInfo * info,
+				  JSValueRef * exception)
 {
   JSObjectRef callback_ref = JSObjectMake (ctx,
 					   seed_callback_class,
@@ -325,18 +340,16 @@ seed_gi_importer_handle_callback (JSContextRef ctx,
 static void
 seed_gi_importer_handle_constant (JSContextRef ctx,
 				  JSObjectRef namespace_ref,
-				  GIConstantInfo *info,
-				  JSValueRef *exception)
+				  GIConstantInfo * info,
+				  JSValueRef * exception)
 {
   GArgument argument;
-  GITypeInfo *constant_type =
-    g_constant_info_get_type (info);
+  GITypeInfo *constant_type = g_constant_info_get_type (info);
   JSValueRef constant_value;
 
   g_constant_info_get_value (info, &argument);
   constant_value =
-    seed_gi_argument_make_js (ctx, &argument,
-			      constant_type, exception);
+    seed_gi_argument_make_js (ctx, &argument, constant_type, exception);
   seed_object_set_property (ctx, namespace_ref,
 			    g_base_info_get_name ((GIBaseInfo *) info),
 			    constant_value);
@@ -346,14 +359,14 @@ seed_gi_importer_handle_constant (JSContextRef ctx,
 
 static gchar *
 seed_gi_importer_get_version (JSContextRef ctx,
-			      gchar *namespace,
-			      JSValueRef *exception)
+			      gchar * namespace, JSValueRef * exception)
 {
   JSValueRef version_ref;
   gchar *version = NULL;
 
-  version_ref = seed_object_get_property (ctx, gi_importer_versions, namespace);
-  if (!JSValueIsUndefined(ctx, version_ref))
+  version_ref =
+    seed_object_get_property (ctx, gi_importer_versions, namespace);
+  if (!JSValueIsUndefined (ctx, version_ref))
     version = seed_value_to_string (ctx, version_ref, exception);
 
   return version;
@@ -361,8 +374,7 @@ seed_gi_importer_get_version (JSContextRef ctx,
 
 static JSObjectRef
 seed_gi_importer_do_namespace (JSContextRef ctx,
-			       gchar *namespace,
-			       JSValueRef *exception)
+			       gchar * namespace, JSValueRef * exception)
 {
   GIBaseInfo *info;
   JSObjectRef namespace_ref;
@@ -380,11 +392,11 @@ seed_gi_importer_do_namespace (JSContextRef ctx,
     }
 
   version = seed_gi_importer_get_version (ctx, namespace, exception);
-  if (!g_irepository_require (NULL, namespace,
-			      version, 0, &e))
+  if (!g_irepository_require (NULL, namespace, version, 0, &e))
     {
       seed_make_exception_from_gerror (ctx, exception, e);
       g_error_free (e);
+      g_free (version);
       return NULL;
     }
   g_free (version);
@@ -407,26 +419,36 @@ seed_gi_importer_do_namespace (JSContextRef ctx,
       switch (info_type)
 	{
 	case GI_INFO_TYPE_FUNCTION:
-	  seed_gi_importer_handle_function (ctx, namespace_ref, (GIFunctionInfo *) info, exception);
+	  seed_gi_importer_handle_function (ctx, namespace_ref,
+					    (GIFunctionInfo *) info,
+					    exception);
 	  break;
 	case GI_INFO_TYPE_ENUM:
 	case GI_INFO_TYPE_FLAGS:
-	  seed_gi_importer_handle_enum (ctx, namespace_ref, (GIEnumInfo *) info, exception);
+	  seed_gi_importer_handle_enum (ctx, namespace_ref,
+					(GIEnumInfo *) info, exception);
 	  break;
 	case GI_INFO_TYPE_OBJECT:
-	  seed_gi_importer_handle_object (ctx, namespace_ref, (GIObjectInfo *) info, exception);
+	  seed_gi_importer_handle_object (ctx, namespace_ref,
+					  (GIObjectInfo *) info, exception);
 	  break;
 	case GI_INFO_TYPE_STRUCT:
-	  seed_gi_importer_handle_struct (ctx, namespace_ref, (GIStructInfo *) info, exception);
+	  seed_gi_importer_handle_struct (ctx, namespace_ref,
+					  (GIStructInfo *) info, exception);
 	  break;
 	case GI_INFO_TYPE_UNION:
-	  seed_gi_importer_handle_union (ctx, namespace_ref, (GIUnionInfo *) info, exception);
+	  seed_gi_importer_handle_union (ctx, namespace_ref,
+					 (GIUnionInfo *) info, exception);
 	  break;
 	case GI_INFO_TYPE_CALLBACK:
-	  seed_gi_importer_handle_callback (ctx, namespace_ref, (GICallbackInfo *) info, exception);
+	  seed_gi_importer_handle_callback (ctx, namespace_ref,
+					    (GICallbackInfo *) info,
+					    exception);
 	  break;
 	case GI_INFO_TYPE_CONSTANT:
-	  seed_gi_importer_handle_constant (ctx, namespace_ref, (GIConstantInfo *) info, exception);
+	  seed_gi_importer_handle_constant (ctx, namespace_ref,
+					    (GIConstantInfo *) info,
+					    exception);
 	  break;
 	default:
 	  break;
@@ -434,10 +456,9 @@ seed_gi_importer_do_namespace (JSContextRef ctx,
       g_base_info_unref (info);
     }
 
-  g_hash_table_insert (gi_imports, g_strdup(namespace), namespace_ref);
+  g_hash_table_insert (gi_imports, g_strdup (namespace), namespace_ref);
 
-  jsextension = g_strdup_printf ("imports.extensions.%s",
-				 namespace);
+  jsextension = g_strdup_printf ("imports.extensions.%s", namespace);
   extension_script = JSStringCreateWithUTF8CString (jsextension);
   JSEvaluateScript (ctx, extension_script, NULL, NULL, 0, NULL);
   JSStringRelease (extension_script);
@@ -452,7 +473,7 @@ static JSValueRef
 seed_gi_importer_get_property (JSContextRef ctx,
 			       JSObjectRef object,
 			       JSStringRef property_name,
-			       JSValueRef *exception)
+			       JSValueRef * exception)
 {
   JSObjectRef ret;
   guint len;
@@ -464,12 +485,12 @@ seed_gi_importer_get_property (JSContextRef ctx,
 
   SEED_NOTE (IMPORTER, "seed_gi_importer_get_property with %s", prop);
 
-  if (!strcmp(prop, "versions"))
+  if (!g_strcmp0 (prop, "versions"))
     return gi_importer_versions;
   // Nasty hack
-  else if (!strcmp(prop, "toString"))
+  else if (!g_strcmp0 (prop, "toString"))
     return 0;
-  if (!strcmp (prop, "valueOf")) // HACK
+  if (!g_strcmp0 (prop, "valueOf"))	// HACK
     return NULL;
 
 
@@ -480,8 +501,31 @@ seed_gi_importer_get_property (JSContextRef ctx,
   return ret;
 }
 
+static JSObjectRef
+seed_make_importer_dir (JSContextRef ctx, gchar * path)
+{
+  gchar *init;
+  JSObjectRef dir;
+
+  dir = JSObjectMake (ctx, importer_dir_class, path);
+
+  init = g_build_filename (path, "__init__.js", NULL);
+  if (g_file_test (init, G_FILE_TEST_IS_REGULAR))
+    {
+      SeedScript *s;
+      SEED_NOTE (IMPORTER, "Found __init__.js (%s)", path);
+
+      s = seed_script_new_from_file (ctx, init);
+      seed_evaluate (ctx, s, dir);
+      seed_script_destroy (s);
+    }
+
+  g_free (init);
+  return dir;
+}
+
 static void
-seed_importer_free_search_path (GSList *path)
+seed_importer_free_search_path (GSList * path)
 {
   GSList *walk = path;
 
@@ -496,29 +540,32 @@ seed_importer_free_search_path (GSList *path)
 
 
 GSList *
-seed_importer_get_search_path (JSContextRef ctx,
-			       JSValueRef *exception)
+seed_importer_get_search_path (JSContextRef ctx, JSValueRef * exception)
 {
   GSList *path = NULL;
+  gchar *entry;
   JSValueRef search_path_ref, length_ref;
   guint length, i;
 
   search_path_ref = seed_object_get_property (ctx, importer, "searchPath");
-  if (!JSValueIsObject(ctx, search_path_ref))
+  if (!JSValueIsObject (ctx, search_path_ref))
     {
-      seed_make_exception (ctx, exception, "ArgumentError", "Importer searchPath object is not an array");
+      seed_make_exception (ctx, exception, "ArgumentError",
+			   "Importer searchPath object is not an array");
       return NULL;
     }
 
-  length_ref = seed_object_get_property (ctx, (JSObjectRef) search_path_ref, "length");
+  length_ref =
+    seed_object_get_property (ctx, (JSObjectRef) search_path_ref, "length");
   length = seed_value_to_uint (ctx, length_ref, exception);
 
   for (i = 0; i < length; i++)
     {
       JSValueRef entry_ref;
-      gchar *entry;
 
-      entry_ref = JSObjectGetPropertyAtIndex (ctx, (JSObjectRef) search_path_ref, i, exception);
+      entry_ref =
+	JSObjectGetPropertyAtIndex (ctx, (JSObjectRef) search_path_ref, i,
+				    exception);
       entry = seed_value_to_string (ctx, entry_ref, exception);
 
       path = g_slist_append (path, entry);
@@ -529,10 +576,9 @@ seed_importer_get_search_path (JSContextRef ctx,
 
 static JSObjectRef
 seed_importer_handle_native_module (JSContextRef ctx,
-				    const gchar *dir,
-				    const gchar *file,
-				    JSValueRef *exception)
-
+				    const gchar * dir,
+				    const gchar * file,
+				    JSValueRef * exception)
 {
   GModule *module;
   JSObjectRef module_obj;
@@ -554,13 +600,12 @@ seed_importer_handle_native_module (JSContextRef ctx,
       // Could be a better exception
       seed_make_exception (ctx, exception, "ModuleError",
 			   "Error loading native module at %s: %s",
-			   file_path,
-			   g_module_error());
+			   file_path, g_module_error ());
       g_free (file_path);
 
       return NULL;
     }
-  g_module_symbol (module, "seed_module_init", (gpointer *) &init);
+  g_module_symbol (module, "seed_module_init", (gpointer *) & init);
   module_obj = (*init) (eng);
   g_hash_table_insert (file_imports, file_path, module_obj);
   SEED_NOTE (IMPORTER, "Loaded native module");
@@ -571,7 +616,7 @@ seed_importer_handle_native_module (JSContextRef ctx,
 }
 
 static gchar *
-seed_importer_canonicalize_path (gchar *path)
+seed_importer_canonicalize_path (gchar * path)
 {
   GFile *file;
   gchar *absolute_path;
@@ -585,16 +630,17 @@ seed_importer_canonicalize_path (gchar *path)
 
 static JSObjectRef
 seed_importer_handle_file (JSContextRef ctx,
-			   const gchar *dir,
-			   const gchar *file,
-			   JSValueRef *exception)
+			   const gchar * dir,
+			   const gchar * file, JSValueRef * exception)
 {
   JSContextRef nctx;
+  JSValueRef js_file_dirname;
   JSObjectRef global, c_global;
   JSStringRef file_contents, file_name;
-  gchar *contents, *walk, *file_path, *canonical;
+  gchar *contents, *walk, *file_path, *canonical, *absolute_path;
+  gchar *normalized_path;
 
-  file_path = g_strconcat (dir, "/", file, NULL);
+  file_path = g_build_filename (dir, file, NULL);
   canonical = seed_importer_canonicalize_path (file_path);
   SEED_NOTE (IMPORTER, "Trying to import file: %s", file_path);
 
@@ -612,7 +658,7 @@ seed_importer_handle_file (JSContextRef ctx,
       if (g_file_test (file_path, G_FILE_TEST_IS_DIR))
 	{
 	  SEED_NOTE (IMPORTER, "File is directory");
-	  return JSObjectMake (ctx, importer_dir_class, file_path);
+	  return seed_make_importer_dir (ctx, file_path);
 	}
       return NULL;
     }
@@ -638,14 +684,30 @@ seed_importer_handle_file (JSContextRef ctx,
   c_global = JSContextGetGlobalObject (ctx);
   JSValueProtect (eng->context, global);
 
+  absolute_path = g_path_get_dirname (file_path);
+  if (!g_path_is_absolute (absolute_path))
+    {
+      g_free (absolute_path);
+      absolute_path = g_build_filename (g_get_current_dir (),
+					g_path_get_dirname (file_path), NULL);
+    }
+
+  normalized_path = canonicalize_file_name (absolute_path);
+
+  js_file_dirname = seed_value_from_string (ctx, normalized_path, NULL);
+
+  seed_object_set_property (nctx, global, "__script_path__", js_file_dirname);
+
   g_hash_table_insert (file_imports, canonical, global);
   g_free (file_path);
+  g_free (absolute_path);
 
   JSEvaluateScript (nctx, file_contents, NULL, file_name, 0, exception);
 
   // Does leak...but it's a debug statement.
   SEED_NOTE (IMPORTER, "Evaluated file, exception: %s",
-	     *exception ? seed_exception_to_string (ctx, *exception) : "(null)");
+	     *exception ? seed_exception_to_string (ctx,
+						    *exception) : "(null)");
 
   JSGlobalContextRelease ((JSGlobalContextRef) nctx);
 
@@ -657,23 +719,27 @@ seed_importer_handle_file (JSContextRef ctx,
 }
 
 static JSObjectRef
-seed_importer_search (JSContextRef ctx,
-		      gchar *prop,
-		      JSValueRef *exception)
+seed_importer_search (JSContextRef ctx, gchar * prop, JSValueRef * exception)
 {
+
+  GDir *dir;
+  GError *e;
+  gchar *mentry;
   GSList *path, *walk;
-  gchar *prop_as_lib = g_strconcat ("libseed_", prop, ".", G_MODULE_SUFFIX, NULL);
+  JSObjectRef ret;
+  gchar *prop_as_lib =
+    g_strconcat ("libseed_", prop, ".", G_MODULE_SUFFIX, NULL);
+  gsize i, mentrylen;
 
   path = seed_importer_get_search_path (ctx, exception);
 
   walk = path;
   while (walk)
     {
-      GError *e = NULL;
-      GDir *dir;
+      e = NULL;
       const gchar *entry;
 
-      dir = g_dir_open ((gchar *)walk->data, 0, &e);
+      dir = g_dir_open ((gchar *) walk->data, 0, &e);
       if (e)
 	{
 	  g_error_free (e);
@@ -681,21 +747,20 @@ seed_importer_search (JSContextRef ctx,
 	  walk = walk->next;
 	  continue;
 	}
-      while ((entry = g_dir_read_name(dir)))
+      while ((entry = g_dir_read_name (dir)))
 	{
-	  guint i;
-	  gchar *mentry = g_strdup (entry);
+	  mentry = g_strdup (entry);
 
-	  for (i = 0; i < strlen (mentry); i++)
+	  mentrylen = strlen (mentry);
+	  for (i = 0; i < mentrylen; i++)
 	    {
 	      if (mentry[i] == '.')
 		mentry[i] = '\0';
 	    }
-	  if (!strcmp (mentry, prop))
+	  if (!g_strcmp0 (mentry, prop))
 	    {
-	      JSObjectRef ret;
-
-	      ret = seed_importer_handle_file (ctx, walk->data, entry, exception);
+	      ret =
+		seed_importer_handle_file (ctx, walk->data, entry, exception);
 
 	      g_dir_close (dir);
 	      g_free (mentry);
@@ -704,11 +769,11 @@ seed_importer_search (JSContextRef ctx,
 
 	      return ret;
 	    }
-	  else if (!strcmp(entry, prop_as_lib))
+	  else if (!g_strcmp0 (entry, prop_as_lib))
 	    {
-	      JSObjectRef ret;
-
-	      ret = seed_importer_handle_native_module (ctx, walk->data, entry, exception);
+	      ret =
+		seed_importer_handle_native_module (ctx, walk->data, entry,
+						    exception);
 	      g_dir_close (dir);
 	      g_free (mentry);
 	      g_free (prop_as_lib);
@@ -732,8 +797,7 @@ seed_importer_search (JSContextRef ctx,
 static JSValueRef
 seed_importer_get_property (JSContextRef ctx,
 			    JSObjectRef object,
-			    JSStringRef property_name,
-			    JSValueRef *exception)
+			    JSStringRef property_name, JSValueRef * exception)
 {
   JSValueRef ret;
   guint len;
@@ -743,11 +807,11 @@ seed_importer_get_property (JSContextRef ctx,
   prop = g_alloca (len * sizeof (gchar));
   JSStringGetUTF8CString (property_name, prop, len);
 
-  if (!strcmp (prop, "gi"))
+  if (!g_strcmp0 (prop, "gi"))
     return gi_importer;
-  if (!strcmp (prop, "searchPath"))
+  if (!g_strcmp0 (prop, "searchPath"))
     return NULL;
-  if (!strcmp (prop, "toString")) // HACK
+  if (!g_strcmp0 (prop, "toString"))	// HACK
     return NULL;
 
   ret = seed_importer_search (ctx, prop, exception);
@@ -759,14 +823,15 @@ static JSValueRef
 seed_importer_dir_get_property (JSContextRef ctx,
 				JSObjectRef object,
 				JSStringRef property_name,
-				JSValueRef *exception)
+				JSValueRef * exception)
 {
   GError *e = NULL;
   GDir *dir;
-  guint len;
+  guint len, i;
+  gsize mentrylen;
   const gchar *entry;
-  gchar *dir_path, *prop;
-
+  gchar *dir_path, *prop, *mentry;
+  JSObjectRef ret;
 
   dir_path = JSObjectGetPrivate (object);
 
@@ -785,18 +850,17 @@ seed_importer_dir_get_property (JSContextRef ctx,
     }
   while ((entry = g_dir_read_name (dir)))
     {
-      gchar *mentry = g_strdup (entry);
-      guint i;
+      mentry = g_strdup (entry);
+      mentrylen = strlen (mentry);
 
-      for (i = 0; i < strlen (mentry); i++)
+      for (i = 0; i < mentrylen; i++)
 	{
 	  if (mentry[i] == '.')
 	    mentry[i] = '\0';
 	}
 
-      if (!strcmp (mentry, prop))
+      if (!g_strcmp0 (mentry, prop))
 	{
-	  JSObjectRef ret;
 
 	  ret = seed_importer_handle_file (ctx, dir_path, entry, exception);
 	  g_dir_close (dir);
@@ -818,19 +882,19 @@ seed_importer_dir_finalize (JSObjectRef dir)
   g_free (dir_path);
 }
 
-
 void
-seed_importer_add_global(JSObjectRef global,
-			 gchar *name)
+seed_importer_add_global (JSObjectRef global, gchar * name)
 {
   JSValueProtect (eng->context, global);
-  g_hash_table_insert (file_imports, seed_importer_canonicalize_path (name), global);
+  g_hash_table_insert (file_imports, seed_importer_canonicalize_path (name),
+		       global);
 }
 
 static void
 seed_importer_dir_enumerate_properties (JSContextRef ctx,
 					JSObjectRef object,
-					JSPropertyNameAccumulatorRef propertyNames)
+					JSPropertyNameAccumulatorRef
+					propertyNames)
 {
   const gchar *entry;
   GDir *dir;
@@ -841,7 +905,9 @@ seed_importer_dir_enumerate_properties (JSContextRef ctx,
   dir = g_dir_open (path, 0, &e);
   if (e)
     {
-      SEED_NOTE(IMPORTER, "Error in g_dir_open in seed_importer_enumerate_dir_properties: %s", e->message);
+      SEED_NOTE (IMPORTER,
+		 "Error in g_dir_open in seed_importer_enumerate_dir_properties: %s",
+		 e->message);
       g_error_free (e);
       // Not much we can do here.
       return;
@@ -861,16 +927,16 @@ seed_importer_dir_enumerate_properties (JSContextRef ctx,
 JSObjectRef
 seed_importer_construct_dir (JSContextRef ctx,
 			     JSObjectRef constructor,
-			     size_t argumentCount,
+			     gsize argumentCount,
 			     const JSValueRef arguments[],
-			     JSValueRef *exception)
+			     JSValueRef * exception)
 {
   gchar *path;
   if (argumentCount != 1)
     {
       seed_make_exception (ctx, exception, "ArgumentError",
 			   "Directory constructor expects 1 argument");
-      return (JSObjectRef)JSValueMakeUndefined (ctx);
+      return (JSObjectRef) JSValueMakeUndefined (ctx);
     }
   path = seed_value_to_string (ctx, arguments[0], exception);
 
@@ -879,22 +945,24 @@ seed_importer_construct_dir (JSContextRef ctx,
       seed_make_exception (ctx, exception, "ArgumentError",
 			   "Path (%s) is not a directory", path);
       g_free (path);
-      return (JSObjectRef)JSValueMakeUndefined (ctx);
+      return (JSObjectRef) JSValueMakeUndefined (ctx);
     }
 
-  return JSObjectMake (ctx, importer_dir_class, path);
+  return seed_make_importer_dir (ctx, path);
 }
 
 void
-seed_importer_set_search_path(JSContextRef ctx,
-			      gchar **search_path)
+seed_importer_set_search_path (JSContextRef ctx, gchar ** search_path)
 {
   JSObjectRef imports, array;
   JSValueRef *array_elem;
   guint length = g_strv_length (search_path), i;
 
   array_elem = g_alloca (length * sizeof (array_elem));
-  imports = (JSObjectRef) seed_object_get_property (ctx, JSContextGetGlobalObject (ctx), "imports");
+  imports =
+    (JSObjectRef) seed_object_get_property (ctx,
+					    JSContextGetGlobalObject (ctx),
+					    "imports");
 
   for (i = 0; i < length; i++)
     {
@@ -909,19 +977,19 @@ seed_importer_set_search_path(JSContextRef ctx,
 JSClassDefinition importer_class_def = {
   0,				/* Version, always 0 */
   0,
-  "importer",		/* Class Name */
+  "importer",			/* Class Name */
   NULL,				/* Parent Class */
   NULL,				/* Static Values */
   NULL,				/* Static Functions */
-  NULL,                         /* Initialize */
+  NULL,				/* Initialize */
   NULL,				/* Finalize */
   NULL,				/* Has Property */
   seed_importer_get_property,	/* Get Property */
-  NULL,                         /* Set Property */
+  NULL,				/* Set Property */
   NULL,				/* Delete Property */
   NULL,				/* Get Property Names */
   NULL,				/* Call As Function */
-  NULL,	/* Call As Constructor */
+  NULL,				/* Call As Constructor */
   NULL,				/* Has Instance */
   NULL				/* Convert To Type */
 };
@@ -933,15 +1001,15 @@ JSClassDefinition gi_importer_class_def = {
   NULL,				/* Parent Class */
   NULL,				/* Static Values */
   NULL,				/* Static Functions */
-  NULL,                         /* Initialize */
+  NULL,				/* Initialize */
   NULL,				/* Finalize */
   NULL,				/* Has Property */
-  seed_gi_importer_get_property,/* Get Property */
+  seed_gi_importer_get_property,	/* Get Property */
   NULL,				/* Set Property */
   NULL,				/* Delete Property */
   NULL,				/* Get Property Names */
   NULL,				/* Call As Function */
-  NULL,	/* Call As Constructor */
+  NULL,				/* Call As Constructor */
   NULL,				/* Has Instance */
   NULL				/* Convert To Type */
 };
@@ -953,21 +1021,21 @@ JSClassDefinition importer_dir_class_def = {
   NULL,				/* Parent Class */
   NULL,				/* Static Values */
   NULL,				/* Static Functions */
-  NULL,                         /* Initialize */
+  NULL,				/* Initialize */
   seed_importer_dir_finalize,	/* Finalize */
   NULL,				/* Has Property */
   seed_importer_dir_get_property,	/* Get Property */
   NULL,				/* Set Property */
   NULL,				/* Delete Property */
-  seed_importer_dir_enumerate_properties,				/* Get Property Names */
+  seed_importer_dir_enumerate_properties,	/* Get Property Names */
   NULL,				/* Call As Function */
-  NULL,	/* Call As Constructor */
+  NULL,				/* Call As Constructor */
   NULL,				/* Has Instance */
   NULL				/* Convert To Type */
 };
 
-void seed_initialize_importer(JSContextRef ctx,
-			      JSObjectRef global)
+void
+seed_initialize_importer (JSContextRef ctx, JSObjectRef global)
 {
   JSObjectRef dir_constructor;
 
@@ -988,7 +1056,8 @@ void seed_initialize_importer(JSContextRef ctx,
 
   /* Passing nonnull for class requires a webkit fix that most people wont have yet. It also has minimal benefit */
   //  dir_constructor = JSObjectMakeConstructor (ctx, importer_dir_class, seed_importer_construct_dir);
-  dir_constructor = JSObjectMakeConstructor (ctx, NULL, seed_importer_construct_dir);
+  dir_constructor =
+    JSObjectMakeConstructor (ctx, NULL, seed_importer_construct_dir);
   seed_object_set_property (ctx, importer, "Directory", dir_constructor);
 
   seed_object_set_property (ctx, global, "imports", importer);
